package com.chelvc.cloud.vehicle.client;

import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.HelpDTO;
import com.chelvc.cloud.vehicle.client.param.HelpModifyParam;
import com.chelvc.cloud.vehicle.client.param.HelpPagingParam;
import com.chelvc.framework.common.model.Pagination;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 帮助Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
@FeignClient("vehicle")
public interface HelpClient {
    /**
     * 查询帮助
     *
     * @param id 帮助主键
     * @return 帮助
     */
    @GetMapping("/help/selectHelpById")
    HelpDTO selectHelpById(@RequestParam("id") Long id);

    /**
     * 查询帮助列表
     *
     * @param param 帮助
     * @return 帮助集合
     */
    @GetMapping("/help/selectHelpList")
    List<HelpDTO> selectHelpList(@SpringQueryMap HelpModifyParam param);

    /**
     * 新增帮助
     *
     * @param param 帮助
     * @return 结果
     */
    @PostMapping("/help/insertHelp")
    void insertHelp(@RequestBody HelpModifyParam param);

    /**
     * 修改帮助
     *
     * @param param 帮助
     * @return 结果
     */
    @PutMapping("/help/updateHelp")
    void updateHelp(@RequestParam("id") Long id, @RequestBody HelpModifyParam param);


    /**
     * 删除帮助信息
     *
     * @param id 帮助主键
     * @return 结果
     */
    @DeleteMapping("/help")
    void deleteHelpById(@RequestParam("id") Long id);

    /**
     * 分页查询帮助列表
     *
     * @param param
     * @return
     */
    @GetMapping("/help/getHelpPaging")
    Pagination<HelpDTO> getHelpPaging(@SpringQueryMap HelpPagingParam param);
}
