package com.chelvc.cloud.vehicle.client;


import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.UserCouponDTO;
import com.chelvc.cloud.vehicle.client.param.UserCouponModifyParam;
import com.chelvc.cloud.vehicle.client.param.UserCouponPagingParam;
import com.chelvc.cloud.vehicle.client.param.UserCouponQueryParam;
import com.chelvc.framework.common.model.Pagination;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 优惠券领取记录业务接口
 *
 * @author 七仔
 * @date 2023/3/29 21:59
 */
@FeignClient("vehicle")
public interface UserCouponClient {
    /**
     * 查询用户优惠券
     *
     * @param param 查询参数
     * @return 用户优惠券信息列表
     */
    @GetMapping("/userCoupon/listUserCoupons")
    List<UserCouponDTO> listUserCoupons(@SpringQueryMap UserCouponQueryParam param);

    /**
     * 获取当前用户对于当前商品可使用的优惠券列表
     *
     * @param param 查询参数
     * @return 用户可用优惠券信息列表
     */
    @GetMapping("/userCoupon/listUserCanUseCoupons")
    List<UserCouponDTO> listUserCanUseCoupons(@SpringQueryMap UserCouponQueryParam param);

    /**
     * 新增优惠券领取记录
     *
     * @param param 新增参数
     * @return 优惠券领取记录主键
     */
    @PostMapping("/userCoupon/addUserCoupon")
    Long addUserCoupon(@RequestBody UserCouponModifyParam param);

    /**
     * 修改优惠券领取记录
     *
     * @param id    优惠券领取记录主键
     * @param param 修改参数
     */
    @PutMapping("/userCoupon/updateUserCoupon")
    void updateUserCoupon(@RequestParam("id") Long id, @RequestBody UserCouponModifyParam param);

    /**
     * 获取优惠券领取记录
     *
     * @param id 优惠券领取记录主键
     * @return 优惠券领取记录
     */
    @GetMapping("/userCoupon/getUserCoupon")
    UserCouponDTO getUserCoupon(@RequestParam("id") Long id);

    /**
     * 查询优惠券领取记录
     *
     * @param param 查询参数
     * @return 优惠券领取记录分页信息
     */
    @GetMapping("/userCoupon/getUserCouponPaging")
    Pagination<UserCouponDTO> getUserCouponPaging(@SpringQueryMap UserCouponPagingParam param);

    /**
     * 领取优惠券
     *
     * @param couponId 优惠券ID
     * @return 优惠券领取记录主键
     */
    @PostMapping("/userCoupon/claimCoupon")
    Long claimCoupon(@RequestParam("couponId") Long couponId);
}
