package com.chelvc.cloud.vehicle.client.param;
import com.chelvc.framework.common.model.File;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 商家认证数据模型
 *
 * @author liude
 * @data 2023/12/10
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
public class MerchantAuthModifyParam implements Serializable {
    /**
     * 主键ID
     */
    private Long id;
    /**
     * 营业执照编号
     */
//    @NotEmpty(message = "营业执照编号不能为空")
    @Size(max = 100, message = "营业执照编号长度不能大于64")
    private String businessLicenseId;
    /**
     * 营业执照证件号
     */
//    @NotEmpty(message = "营业执照证件号不能为空")
    @Size(max = 64, message = "营业执照证件号长度不能大于64")
    private String businessLicenseNumber;
    /**
     * 营业执照有效期
     */
    @NotNull(message = "营业执照有效期不能为空")
    private Date businessLicenseExpirationDate;
    /**
     * 身份证号
     */
    @NotEmpty(message = "身份证号不能为空")
    @Size(max = 32, message = "身份证号长度不能大于32")
    private String idCardNumber;
    /**
     * 身份证有效期开始时间
     */
    @NotNull(message = "身份证有效期开始时间不能为空")
    private Date idCardExpirationDateStart;
    /**
     * 身份证有效期
     */
    @NotNull(message = "身份证有效期不能为空")
    private Date idCardExpirationDate;
    /**
     * 店铺图
     */
    private String storeImage;
    /**
     * 法人姓名
     */
    @NotEmpty(message = "法人姓名不能为空")
    @Size(max = 255, message = "法人姓名长度t不能大于255")
    private String legalRepresentativeName;
    /**
     * 法人性别
     */
//    @NotEmpty(message = "法人性别不能为空")
    private String legalRepresentativeGender;
    /**
     * 手机号
     */
    @NotEmpty(message = "手机号不能为空")
    @Size(max = 20, message = "手机号长度不能大于20")
    private String mobileNumber;
    /**
     * 账号
     */
    @Size(max = 64, message = "账号长度不能大于64")
    private String account;
    /**
     * 账号注册日期
     */
    private Date accountRegistrationDate;
    /**
     * 店铺类型
     */
    private String storeType;
    /**
     *店名
     */
    @NotEmpty(message = "店名不能为空")
    @Size(max = 255, message = "店名长度不能大于255")
    private String storeName;
    /**
     * 店铺地址
     */
//    @NotNull(message = "所属地区不能为空")
//    @Min(value = 100000, message = "所属地区不能小于100000")
//    @Max(value = 999999, message = "所属地区不能大于999999")
    private String storeAddress;
    /**
     * 经营许可证照片
     */
    private String businessLicensePhoto;
    /**
     * 经营许可证号
     */
//    @NotEmpty(message = "经营许可证号不能为空")
    @Size(max = 20, message = "经营许可证号长度不能大于20")
    private String businessLicenseNumberValid;
    /**
     * 经营许可证有效期
     */
//    @NotNull(message = "经营许可证有效期不能为空")
    private Date businessLicenseExpirationDateValid;
    /**
     * 推荐人
     */
    private String recommendationUser;

    /**
     * 营业执照证件号
     */
    private String businessPhoto;
    /**
     * 身份证正面照片
     */
    private String idCardFrontPhoto;

    /**
     * 身份证反面照片
     */
    private String idCardBackPhoto;

    /**
     * 店铺内部环境图
     */
    private List<File> storeInnerImage;


    /**
     * 店铺详细地址
     */
    @Size(max = 255, message = "店铺详细地址长度不能大于255")
    private String storeAddressDetail;

    /**
     * 邮箱
     */
    @Size(max = 255, message = "邮箱长度不能大于255")
    private String email;

    /**
     * 所属地区
     */
    @NotNull(message = "所属地区不能为空")
    @Min(value = 100000, message = "所属地区不能小于100000")
    @Max(value = 999999, message = "所属地区不能大于999999")
    private Integer region;


    /**
     * 营业时间
     */
    @NotEmpty(message = "营业时间不能为空")
    @Size(max = 50, message = "营业时间长度不能大于50")
    private String opening;

    /**
     * 省
     */
    @NotNull(message = "所属省份不能为空")
    @Min(value = 100000, message = "所属省份不能小于100000")
    @Max(value = 999999, message = "所属省份不能大于999999")
    private Integer province;

    /**
     * 市
     */
    @NotNull(message = "所属城市不能为空")
    @Min(value = 100000, message = "所属城市不能小于100000")
    @Max(value = 999999, message = "所属城市不能大于999999")
    private Integer city;

    /**
     * 区
     */
    @NotNull(message = "所在地区不能为空")
    @Min(value = 100000, message = "所属区不能小于100000")
    @Max(value = 999999, message = "所属区不能大于999999")
    private Integer district;

    /**
     * 经度
     */
    @NotNull(message = "经度不能为空")
    private Double longitude;

    /**
     * 纬度
     */
    @NotNull(message = "纬度不能为空")
    private Double latitude;

    /**
     * 开户账号
     */
    @NotNull(message = "开户账号不能为空")
    private String bankCardNo;
    /**
     * 银行预留手机号
     */
    @NotNull(message = "银行预留手机号不能为空")
    private String reservedPhoneNo;
    /**
     * 营业执照名称
     */
    @NotNull(message = "营业执照名称不能为空")
    private String businessName;
    /**
     * 商家主营业务
     */
    private Long mainBusiness;
    /**
     * 商家副营业务
     */
    private List<Long> ancillaryBusiness;

    /**
     * 营业状态 0-营业中 1-休息中
     */
    private String businessStatus;
}
