package com.chelvc.cloud.vehicle.server.controller;

import java.util.List;
import java.util.Map;

import com.chelvc.cloud.vehicle.client.dto.AttentionTotalDTO;
import com.chelvc.cloud.vehicle.client.dto.AttentionUserDTO;
import com.chelvc.cloud.vehicle.client.param.QueryAttentionParam;
import com.chelvc.cloud.vehicle.server.service.AttentionService;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author: igl
 * @date: 2023/8/14 10:11
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AttentionController {

    private final AttentionService attentionService;

    /**
     * 关注用户
     *
     * @param userId
     * @param targetUserId
     */
    @PostMapping("/attention/concern")
    public void concern(@RequestParam("id") Long userId, @RequestParam("targetUserId") Long targetUserId) {
        attentionService.concern(userId, targetUserId);
    }

    /**
     * 分页查看关注、粉丝、好友
     *
     * @param page
     * @param param
     * @author igl
     * @date 2023/2/25 19:42
     */
    @GetMapping("/attention/page")
    public Pagination<AttentionUserDTO> page(@RequestParam("page") Paging page,
                                             @SpringQueryMap QueryAttentionParam param,
                                             @RequestParam("userId") Long userId) {
        return attentionService.page(page, param, userId);
    }

    /**
     * 查询用户是否关注
     *
     * @param userId
     * @param userIds
     * @return
     */
    @GetMapping("/attention/queryAttentionByUserIds")
    public Map<Long, Boolean> queryAttentionByUserIds(@RequestParam("userId") Long userId,
                                                      @RequestParam("userIds") List<Long> userIds) {
        return attentionService.queryAttentionByUserIds(userId, userIds);
    }

    /**
     * 查看关注、粉丝、好友总数
     *
     * @author igl
     * @date 2023/8/15 19:42
     */
    @GetMapping("/attention/queryTotalByUserId")
    public AttentionTotalDTO queryTotalByUserId(@RequestParam("userId") Long userId) {
        return attentionService.queryTotalByUserId(userId);
    }
}
