package com.chelvc.cloud.vehicle.server.controller;

import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.CommissionConfigDTO;
import com.chelvc.cloud.vehicle.client.param.CommissionConfigModifyParam;
import com.chelvc.cloud.vehicle.client.param.CommissionConfigPagingParam;
import com.chelvc.cloud.vehicle.server.service.CommissionConfigService;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 邀请别人消费，邀请人数的不同抽成配置接口
 *
 * @author liude
 * @data 2023/12/31
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class CommissionConfigController {

    private final CommissionConfigService commissionConfigService;

    /**
     * 新增抽成配置
     *
     * @param param 新增参数
     * @return 抽成配置主键
     */
    @PostMapping("/commissionConfig/addCommissionConfig")
    public Long addCommissionConfig(@RequestBody CommissionConfigModifyParam param) {
        return commissionConfigService.addCommissionConfig(param);
    }

    /**
     * 修改抽成配置
     *
     * @param id    抽成配置主键
     * @param param 修改参数
     */
    @PutMapping("/commissionConfig/updateCommissionConfig")
    public void updateCommissionConfig(@RequestParam("id") Long id, @RequestBody CommissionConfigModifyParam param) {
        commissionConfigService.updateCommissionConfig(id, param);
    }

    /**
     * 查询抽成配置
     *
     * @param param 查询参数
     * @return 抽成配置分页信息
     */
    @GetMapping("/commissionConfig/getCommissionConfigPaging")
    public Pagination<CommissionConfigDTO> getCommissionConfigPaging(CommissionConfigPagingParam param) {
        return commissionConfigService.getCommissionConfigPaging(param);
    }

    /**
     * 删除抽成配置
     *
     * @param id 抽成配置主键
     */
    @DeleteMapping("/commissionConfig")
    public void deleteCommissionConfig(@RequestParam("id") Long id) {
        commissionConfigService.deleteCommissionConfig(id);
    }

    /**
     * 分佣配置列表
     *
     * @return
     */
    @GetMapping("/commissionConfig/listCommissionConfig")
    public List<CommissionConfigDTO> listCommissionConfig() {
        return commissionConfigService.listCommissionConfig();
    }
}
