package com.chelvc.cloud.vehicle.server.controller;


import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.OmsOrderSettingDTO;
import com.chelvc.cloud.vehicle.client.param.OmsOrderSettingModifyParam;
import com.chelvc.cloud.vehicle.server.service.OmsOrderSettingService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 订单设置Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OmsOrderSettingController {
    private final OmsOrderSettingService omsOrderSettingService;

    /**
     * 查询订单设置
     *
     * @param id 订单设置主键
     * @return 订单设置
     */
    @GetMapping("/omsOrderSetting/selectOmsOrderSettingById")
    public OmsOrderSettingDTO selectOmsOrderSettingById(@RequestParam("id") Long id) {
        return omsOrderSettingService.selectOmsOrderSettingById(id);
    }

    /**
     * 查询订单设置列表
     *
     * @param omsOrderSetting 订单设置
     * @return 订单设置集合
     */
    @GetMapping("/omsOrderSetting/selectOmsOrderSettingList")
    public List<OmsOrderSettingDTO> selectOmsOrderSettingList(OmsOrderSettingModifyParam omsOrderSetting) {
        return omsOrderSettingService.selectOmsOrderSettingList(omsOrderSetting);
    }

    /**
     * 新增订单设置
     *
     * @param omsOrderSetting 订单设置
     * @return 结果
     */
    @PostMapping("/omsOrderSetting/insertOmsOrderSetting")
    public int insertOmsOrderSetting(@RequestBody OmsOrderSettingModifyParam omsOrderSetting) {
        return omsOrderSettingService.insertOmsOrderSetting(omsOrderSetting);
    }

    /**
     * 修改订单设置
     *
     * @param omsOrderSetting 订单设置
     * @return 结果
     */
    @PostMapping("/omsOrderSetting/updateOmsOrderSetting")
    public int updateOmsOrderSetting(@RequestBody OmsOrderSettingModifyParam omsOrderSetting) {
        return omsOrderSettingService.updateOmsOrderSetting(omsOrderSetting);
    }
}
