package com.chelvc.cloud.vehicle.server.controller;

import com.chelvc.cloud.vehicle.client.dto.TeamDTO;
import com.chelvc.cloud.vehicle.client.param.TeamInvitePageParam;
import com.chelvc.cloud.vehicle.client.param.UserInviteParams;
import com.chelvc.cloud.vehicle.server.service.UserInviteService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UserInviteController {

    private final UserInviteService userInviteService;

    @PostMapping("/userInvite/bindInvite")
    public void bindInvite(@RequestBody UserInviteParams params) {
        userInviteService.bindInvite(params);
    }

    @PostMapping("/invite/unbind")
    public void unbind(@RequestParam("id") Long id) {
        userInviteService.unbind(id);
    }

    @GetMapping("/userInvite/teamList")
    public TeamDTO teamList(@SpringQueryMap TeamInvitePageParam param, @RequestParam("userId") Long userId) {
        return userInviteService.teamList(param, userId);
    }

    /**
     * 查看上级用户
     *
     * @param userId
     * @return
     */
    @GetMapping("/userInvite/queryUserByTarget")
    public Long queryUserByTarget(@RequestParam("userId") Long userId, @RequestParam("type") Integer type) {
        return userInviteService.queryUserByTarget(userId, type);
    }

    /**
     * 查询用户邀请得用户列表
     *
     * @param param
     * @return
     */
    @GetMapping("/userInvite/inviteList")
    public TeamDTO inviteList(TeamInvitePageParam param) {
        return userInviteService.inviteList(param);
    }
}
