package com.chelvc.cloud.vehicle.server.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.chelvc.cloud.vehicle.client.dto.MerchantCartItemDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsCartItemDTO;
import com.chelvc.cloud.vehicle.client.param.OmsCartItemModifyParam;
import com.chelvc.cloud.vehicle.server.entity.OmsCartItem;

import java.util.List;

/**
 * 购物车Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
public interface OmsCartItemService extends IService<OmsCartItem>{

    /**
     * 获取包含促销活动信息的购物车列表
     */
    List<OmsCartItemDTO> listPromotion(List<Long> cartIds);

    /**
     * 批量删除购物车中的商品
     */
    void delete(List<Long> ids,Long userId);

    /**
     * 根据商家查询订单
     * @param merchantId
     * @return
     */
    public List<OmsCartItemDTO> merchantListCartItem(Long merchantId);

    /**
     * 查询购物车中是否包含该商品，有增加数量，无添加到购物车
     */
    int add(OmsCartItemModifyParam cartItem);

    /**
     * 根据会员编号获取购物车列表
     */
    List<OmsCartItemDTO> listCartItem();


    /**
     * 修改某个购物车商品的数量
     */
    int updateQuantity(Long id, Integer quantity);

    /**
     * 批量删除购物车中的商品
     */
    void delete(List<Long> ids);

    /**
     *获取购物车中用于选择商品规格的商品信息
     */
//    CartProduct getCartProduct(Long productId);

    /**
     * 修改购物车中商品的规格
     */
    int updateAttr(OmsCartItemModifyParam cartItem);

    /**
     * 清空购物车
     */
    void clear();

    /**
     * 根据会员编号及商家获取购物车列表
     */
    MerchantCartItemDTO listCartItem(Long merchantId);


    /**
     * 根据会员编号及商家获取购物车列表
     */
//    List<MerchantCartItemDTO> listCartItemByUserId();
}
