package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.client.param.LocationQueryParam;
import com.chelvc.cloud.vehicle.server.dao.HotMerchantMapper;
import com.chelvc.cloud.vehicle.server.entity.HotMerchant;
import com.chelvc.cloud.vehicle.server.service.CategoryService;
import com.chelvc.cloud.vehicle.server.service.HotMerchantService;
import com.chelvc.framework.base.context.SessionContextHolder;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.validation.Valid;
import java.util.Date;
import java.util.List;

/**
 * 热门搜索业务操作实现
 *
 * @author liude
 * @date 2023/12/16
 */
@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired, @Lazy}))
public class HotMerchantServiceImpl extends ServiceImpl<HotMerchantMapper, HotMerchant> implements HotMerchantService{

    private final CategoryService categoryService;

    @Override
    public Long addHotMerchant(@NonNull Long id) {
        HotMerchant hotMerchant = new HotMerchant();
        hotMerchant.setMerchantId(id);
        Long userId = SessionContextHolder.getId();
        hotMerchant.setUserId(userId);
        hotMerchant.setCreator(userId);
        hotMerchant.setCreateTime(new Date());
        hotMerchant.setCount(1);
        this.save(hotMerchant);
        return hotMerchant.getId();
    }

    @Override
    public List<MerchantDTO> listHotMerchant(LocationQueryParam param){
        return this.baseMapper.listHotMerchant(param);
    }


    @Override
    public List<MerchantDTO> likeMerchant(@Valid LocationQueryParam param){
        Long userId = SessionContextHolder.getId();
        List<MerchantDTO> list = this.baseMapper.likeMerchant(userId,param);
        list.forEach(item ->{
            item.setMainBusinessName(this.categoryService.getById(item.getMainBusiness()).getName());
        });
        return list;
    }
 }
