package com.chelvc.cloud.vehicle.server.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.constant.MerchantApplyStatus;
import com.chelvc.cloud.vehicle.client.param.SxyNotifyParam;
import com.chelvc.cloud.vehicle.server.dao.MerchantApplyRecordMapper;
import com.chelvc.cloud.vehicle.server.entity.MerchantApplyRecord;
import com.chelvc.cloud.vehicle.server.handle.TradeHandle;
import com.chelvc.cloud.vehicle.server.service.MerchantApplyRecordService;
import com.chelvc.cloud.vehicle.server.service.MerchantAuthService;
import com.chelvc.cloud.vehicle.server.service.MerchantRelationService;
import com.chelvc.cloud.vehicle.server.service.UserAuthService;
import com.upay.sdk.Constants;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class MerchantApplyRecordServiceImpl extends
        ServiceImpl<MerchantApplyRecordMapper, MerchantApplyRecord> implements MerchantApplyRecordService {

    private final MerchantRelationService merchantRelationService;
    private final UserAuthService userAuthService;
    private final MerchantAuthService merchantAuthService;

    @Override
    public int insert(MerchantApplyRecord record) {
        return baseMapper.insert(record);
    }

    @Override
    public void updateByNotify(SxyNotifyParam param) {
        JSONObject responseData = TradeHandle.decrypt(param);
        String status = responseData.getString("subMerchantReviewStatus");
        if (Constants.ERROR.equals(status)) {
            //解密后报文，如果status=ERROR时，不需要验签
            return;
        }
        String requestId = responseData.getString("requestId");
        LambdaQueryWrapper<MerchantApplyRecord> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MerchantApplyRecord::getRequestId, requestId);
        MerchantApplyRecord record = baseMapper.selectOne(wrapper);
        record.setExamineStatus(status);
        record.setCertificateSupplementUrl(responseData.getString("certificateSupplementUrl"));
        record.setElectronicContractingUrl(responseData.getString("electronicContractingUrl"));
        record.setMerchantReviewRemarks(responseData.getString("merchantReviewRemarks"));
        record.setPostReviewRemark(responseData.getString("postReviewRemark"));
        record.setPostReviewStatus(responseData.getString("postReviewStatus"));
        String subMerchantId = responseData.getString("subMerchantId");
        record.setSubMerchantId(subMerchantId);
        record.setUpdateTime(new Date());
        baseMapper.updateById(record);
        Integer type = record.getType();
        if(type == 1){
            //更改个人认证状态
            userAuthService.updateStatus(record.getUserId(), MerchantApplyStatus.getStatus(status));
        }
        if(type == 2){
            //更改个人认证状态
            merchantAuthService.updateStatus(record.getUserId(), MerchantApplyStatus.getStatus(status));
        }
        if("SUCCESS".equals(status)){
            merchantRelationService.insert(record.getUserId(), record.getType(), subMerchantId);
        }
    }
}
