package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.constant.PlatformConstant;
import com.chelvc.cloud.vehicle.client.dto.PlatformProfitRatioDTO;
import com.chelvc.cloud.vehicle.client.param.EditPlatformProfitRatioParam;
import com.chelvc.cloud.vehicle.client.param.QueryPlatformProfitRatioParam;
import com.chelvc.cloud.vehicle.server.dao.PlatformProfitRatioMapper;
import com.chelvc.cloud.vehicle.server.entity.PlatformProfitRatio;
import com.chelvc.cloud.vehicle.server.entity.ProfitRatioConfig;
import com.chelvc.cloud.vehicle.server.service.OmsOrderService;
import com.chelvc.cloud.vehicle.server.service.PlatformProfitRatioService;
import com.chelvc.cloud.vehicle.server.service.ProfitRatioConfigService;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired, @Lazy}))
public class PlatformProfitRatioServiceImpl extends ServiceImpl<PlatformProfitRatioMapper, PlatformProfitRatio>
        implements PlatformProfitRatioService {

    private final ProfitRatioConfigService profitRatioConfigService;

    private final OmsOrderService omsOrderService;

    @Override
    @Async
    public void updateProfitRatio() {
        Integer total = baseMapper.selectCount(null);
        if (total <= 0) {
            return;
        }
        //查询平台分佣配置
        List<ProfitRatioConfig> profitRatioConfigs = profitRatioConfigService.queryListByType(1);
        if (CollectionUtils.isEmpty(profitRatioConfigs)) {
            return;
        }
        int pageNum = 1;
        int pageSize = 100;
        int pages = 1;
        if (total > pageSize) {
            pages = total % 100 != 0 ? (total / 100) + 1 : total / 100;
        }
        while (pageNum <= pages) {
            //分页查询平台抽成比例
            Pagination<PlatformProfitRatioDTO> platformProfitRatioDTOPagination =
                    queryPageList(pageNum, pageSize);
            List<PlatformProfitRatioDTO> records = platformProfitRatioDTOPagination.getRecords();
            if (!CollectionUtils.isEmpty(records)) {
                for (PlatformProfitRatioDTO dto : records) {
                    Long merchantId = dto.getMerchantId();
                    //查询该商家上个月营业额
                    BigDecimal amount = omsOrderService.queryAmountByLastMonth(merchantId);
                    for (ProfitRatioConfig config : profitRatioConfigs) {
                        int num = config.getNum();
                        if (amount.compareTo(new BigDecimal(num)) > -1) {
                            baseMapper.updateRatio(dto.getId(), config.getRatio(), LocalDateTime.now(), amount);
                            break;
                        }
                    }
                }
            }
            pageNum += 1;
        }
    }

    @Override
    public int queryRatio(Long merchantId) {
        LambdaQueryWrapper<PlatformProfitRatio> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(PlatformProfitRatio::getMerchantId, merchantId);
        PlatformProfitRatio platformProfitRatio = baseMapper.selectOne(wrapper);
        if (platformProfitRatio != null) {
            return platformProfitRatio.getRatio();
        }
        return init(merchantId);
    }

    @Override
    public Pagination<PlatformProfitRatioDTO> queryPageList(QueryPlatformProfitRatioParam param) {
        Paging paging = param.getPaging();
        Page<PlatformProfitRatioDTO> page = new Page<>(paging.getNumber(), paging.getSize());
        QueryWrapper<PlatformProfitRatioDTO> wrapper = Wrappers.query();
        wrapper.eq(param.getUserId() != null, "t.user_id", param.getUserId());
        wrapper.orderByAsc("t.create_time");
        IPage<PlatformProfitRatioDTO> result = baseMapper.queryPageList(page, wrapper);
        return Pagination.<PlatformProfitRatioDTO>builder().total(result.getTotal()).pages(result.getPages())
                .records(result.getRecords()).build();
    }

    public Pagination<PlatformProfitRatioDTO> queryPageList(Integer number, Integer size) {
        Page<PlatformProfitRatioDTO> page = new Page<>(number, size);
        QueryWrapper<PlatformProfitRatioDTO> wrapper = Wrappers.query();
        wrapper.orderByAsc("t.create_time");
        IPage<PlatformProfitRatioDTO> result = baseMapper.queryPageList(page, wrapper);
        return Pagination.<PlatformProfitRatioDTO>builder().total(result.getTotal()).pages(result.getPages())
                .records(result.getRecords()).build();
    }

    public int init(Long merchantId) {
        ProfitRatioConfig config = profitRatioConfigService.queryByTypeAndLeast(1);
        PlatformProfitRatio ratio = new PlatformProfitRatio();
        ratio.setMerchantId(merchantId);
        ratio.setRatio(config == null ? 10 : config.getRatio());
        ratio.setUpdater(PlatformConstant.PLATFORM_ID);
        ratio.setUpdater(PlatformConstant.PLATFORM_ID);
        ratio.setUpdateTime(LocalDateTime.now());
        ratio.setCreateTime(LocalDateTime.now());
        ratio.setTurnover(new BigDecimal("0"));
        baseMapper.insert(ratio);
        return ratio.getRatio();
    }

    @Override
    public void edit(EditPlatformProfitRatioParam param) {
        Long merchantId = param.getMerchantId();
        LambdaQueryWrapper<PlatformProfitRatio> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(PlatformProfitRatio::getMerchantId, merchantId);
        PlatformProfitRatio platformProfitRatio = baseMapper.selectOne(wrapper);
        if (platformProfitRatio == null) {
            init(merchantId);
            return;
        }
        platformProfitRatio.setRatio(param.getRatio());
        platformProfitRatio.setUpdater(param.getOperateUserId());
        platformProfitRatio.setUpdateTime(LocalDateTime.now());
        baseMapper.updateById(platformProfitRatio);
    }
}
